//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "BackupCpyDlgs.h"
#include "wipeDlgs.h"
#include "Global.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TBackupCpyDlg *BackupCpyDlg;
TCopyThread *CopyThread;
TZapThread *ZapThread;
//---------------------------------------------------------------------------
// ZapThread
//---------------------------------------------------------------------------
void  __fastcall DoThreadZap(AnsiString Filename, int Zapcount)
{
   ZapThread = new TZapThread(true);
   ZapThread->filename = Filename;
   ZapThread->zaps = Zapcount;
   ZapThread->Resume();
}
//---------------------------------------------------------------------------
__fastcall TZapThread::TZapThread(bool CreateSuspended)
	: TThread(CreateSuspended)
{
   FreeOnTerminate = true;
}
//---------------------------------------------------------------------------
void __fastcall TZapThread::Execute()
{
   zapfile(filename,zaps);
}
//---------------------------------------------------------------------------
// CopyThread
//---------------------------------------------------------------------------
void  __fastcall DoThreadCopy(TBackupCpyDlg *Owner, AnsiString Filename, AnsiString Copyname)
{
   CopyThread = new TCopyThread(true);
   CopyThread->owner = Owner;
   CopyThread->filename = Filename;
   CopyThread->copyname = Copyname;
   CopyThread->Resume();
}
//---------------------------------------------------------------------------
__fastcall TCopyThread::TCopyThread(bool CreateSuspended)
	: TThread(CreateSuspended)
{
   FreeOnTerminate = true;
}
//---------------------------------------------------------------------------
void __fastcall TCopyThread::Execute()
{
   if (!CopyFile(filename.c_str(),copyname.c_str(),false))
      errorstr = GetLastErrorStr();
}
//---------------------------------------------------------------------------
void __fastcall TCopyThread::Terminate(TObject *Sender)
{
   owner->errorstr = errorstr;
   if (errorstr.IsEmpty()) owner->ModalResult = mrOk;
   else owner->ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
// Backup Restore Routines
//---------------------------------------------------------------------------
bool __fastcall DoBackupDatabase(TForm *Owner, AnsiString Path, AnsiString dbName,
   int Copies, int Passes)
{
   BackupCpyDlg = new TBackupCpyDlg(Owner);
   bool ok = BackupCpyDlg->ExecuteBackup(Path,dbName,Copies,Passes);
   delete BackupCpyDlg;
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall DoRestoreDatabase(TForm *Owner, AnsiString dbName, AnsiString BakFile)
{
   BackupCpyDlg = new TBackupCpyDlg(Owner);
   bool ok = BackupCpyDlg->ExecuteRestore(dbName,BakFile);
   delete BackupCpyDlg;
   return ok;
}
//---------------------------------------------------------------------------
__fastcall TBackupCpyDlg::TBackupCpyDlg(TComponent* Owner)
	: TForm(Owner)
{
   CenterDialog(this,Owner);
   Display->Caption = "";
   isbackup = true;
}
//---------------------------------------------------------------------------
bool __fastcall TBackupCpyDlg::ExecuteBackup(AnsiString Path, AnsiString Name,
   int Copies, int Passes)
{
   path = Path;
   name = Name;
   copies = Copies;
   passes = Passes;
   isbackup = true;
   return ShowModal()==mrOk;
}
//---------------------------------------------------------------------------
bool __fastcall TBackupCpyDlg::ExecuteRestore(AnsiString Name, AnsiString BakFile)
{
   name = Name;
   bakfile = BakFile;
   isbackup = false;
   return ShowModal()==mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TBackupCpyDlg::Timer1Timer(TObject *Sender)
{
   bool ok;
   Timer1->Enabled = false;
   displaymsg = isbackup ? "...saving...":"...restoring...";
   if (isbackup) ok = BackupDatabase();
   else ok = RestoreDatabase();
   ModalResult = ok ? mrOk:mrCancel;
}
//---------------------------------------------------------------------------
bool __fastcall TBackupCpyDlg::RestoreDatabase()
{
   Snooze(200);
   FlashMessage(displaymsg);
   if (!CopyFile(bakfile.c_str(),name.c_str(),false))
   {
      BingBox(this,
         s("System Error: "+GetLastErrorStr())+s("File: "+bakfile),
         "Copy Failed",MB_OK);
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
void __fastcall TBackupCpyDlg::FlashMessage(AnsiString msg)
{
   Display->Caption = msg; Snooze(300);
   Display->Caption =  ""; Snooze(300);
   Display->Caption = msg; Snooze(300);
   Display->Caption =  ""; Snooze(300);
   Display->Caption = msg; Snooze(300);
}
//---------------------------------------------------------------------------
bool __fastcall TBackupCpyDlg::BackupDatabase()
{
   Snooze(200);
   FlashMessage("...copying...");
   AnsiString bakfile=AddBS(path)+ExtractFileName(name),tmpfile=bak(bakfile,1)+".tmp";
   if (!CopyFile(name.c_str(),tmpfile.c_str(),false))
   {
      BingBox(this,
         s("System Error: "+GetLastErrorStr())+
         s("File: "+bakfile),
         "Copy Failed",
         MB_OK
      );
      return false;
   }

   FlashMessage("...verifying...");
   TMemoryStream *Org = new TMemoryStream();
   TMemoryStream *Bak = new TMemoryStream();
   Bak->LoadFromFile(tmpfile);
   Org->LoadFromFile(name);
   bool flawed = Org->Size!=Bak->Size || memcmp(Org->Memory,Bak->Memory,Org->Size)!=0;
   delete Org;
   delete Bak;
   if (flawed)
   {
      BingBox(this,
         s("Your database has been copied, but failed the verify test.\n")+
         s("I'll remove the file. Please try your backup again."),
         s("File Verify Error"),MB_OK);
      if (passes) wipefiles(tmpfile,passes);
      else DeleteFile(tmpfile.c_str());
      return false;
   }
   // rename each backup copy then
   if (FileExists(bak(bakfile,copies)))
   {
      if (passes)
      {
         FlashMessage("...wiping...");
         zapfile(bak(bakfile,copies),passes);
      }
      else DeleteFile(bak(bakfile,copies).c_str());
   }
   for (int i=copies; i>=1; i--)
      if (FileExists(bak(bakfile,i-1)))
          rename(bak(bakfile,i-1).c_str(),bak(bakfile,i).c_str());
   rename(tmpfile.c_str(),bak(bakfile,1).c_str());

   Hide(); Snooze(300);
   return true;
}
//---------------------------------------------------------------------------
void __fastcall TBackupCpyDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
